@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2003-2015 FreeIPMI Core Team
- * 
+ *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- * 
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -90,7 +90,7 @@ assemble_rmcp_pkt (fiid_obj_t obj_rmcp_hdr,
                    fiid_obj_t obj_cmd,
                    void *pkt,
                    unsigned int pkt_len,
-		   unsigned int flags)
+                   unsigned int flags)
 {
   int obj_cmd_len, obj_rmcp_hdr_len;
   unsigned int flags_mask = 0;
@@ -139,7 +139,6 @@ assemble_rmcp_pkt (fiid_obj_t obj_rmcp_hdr,
       return (-1);
     }
 
-  memset (pkt, '\0', pkt_len);
   if ((obj_rmcp_hdr_len = fiid_obj_get_all (obj_rmcp_hdr,
                                             pkt,
                                             pkt_len)) < 0)
@@ -163,7 +162,7 @@ unassemble_rmcp_pkt (const void *pkt,
                      unsigned int pkt_len,
                      fiid_obj_t obj_rmcp_hdr,
                      fiid_obj_t obj_cmd,
-		     unsigned int flags)
+                     unsigned int flags)
 {
   unsigned int indx = 0;
   int len;
